# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
# Date: 03.09.2016                                                            #
# Author: Ole-Johan Skrede                                                    #
#                                                                             #
# Solution proposal as part of the exercise program in                        #
# INF4300 - Digital image analysis at the University of Oslo                  #
#                                                                             #
# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
"""
Solution proposal for exercise 1, task 1.
"""

# pylint: disable=expression-not-assigned
# pylint: disable=bad-indentation
# pylint: disable=redefined-outer-name

import os

import numpy as np
from scipy import signal # pylint: disable=import-error
import cv2
import matplotlib.pyplot as plt

def plot_image(image, fig_num, name=None, colormap='gray', write_file=None):
  """Plot image"""
  fig = plt.figure(fig_num)
  plt.imshow(image, cmap=colormap, interpolation='none')
  if name is not None:
    plt.title(name)
  plt.xticks([]), plt.yticks([])
  plt.tight_layout()
  if write_file:
    fig.savefig(write_file, bbox_inches='tight', pad_inches=0)
  fig_num += 1
  return fig_num

def main():
  """main"""

print('='*80)
print('Solution to weekly exercises in INF4300')
print('Exercise 1')
print('Task 1')
print('-'*80)

image_dir = '../../images'
write_dir = 'results/images'
fig_num = 0

# Read 2D graylevel image
img_filename = os.path.join(image_dir, 'football.jpg')
img = cv2.imread(img_filename, cv2.IMREAD_GRAYSCALE)
fig_num = plot_image(img, fig_num,
                     write_file=os.path.join(write_dir, 'football.png'))
print('Shape of original:  ', img.shape)

# Construct 5x5 mean kernel
mean_kernel = np.ones((5, 5)) / (5*5)

#### Full padding mode
mean_img_full = signal.convolve2d(img, mean_kernel, mode='full')
fig_num = plot_image(mean_img_full, fig_num,
                     write_file=os.path.join(write_dir, 'mean_full.png'))
print('Shape of convolved, full mode: ', mean_img_full.shape)
# Retrieve original: mean_img = mean_img[2:-2, 2:-2]

#### Valid padding mode
mean_img_valid = signal.convolve2d(img, mean_kernel, mode='valid')
fig_num = plot_image(mean_img_valid, fig_num,
                     write_file=os.path.join(write_dir, 'mean_valid.png'))
print('Shape of convolved, valid mode: ', mean_img_valid.shape)

#### Same padding mode
mean_img_same = signal.convolve2d(img, mean_kernel, mode='same', boundary='symm')
fig_num = plot_image(mean_img_same, fig_num,
                     write_file=os.path.join(write_dir, 'mean_same.png'))
print('Shape of convolved, full mode: ', mean_img_same.shape)


plt.show()

if __name__ == "__main__":
  main()
